/*
Copyright (c) 2009, Miroslav Kulich
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of the Player Project nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef VIEWER_HH
#define VIEWER_HH

#include "stage.hh"
#include <libplayerc++/playerc++.h>
#include <pthread.h>

using namespace Stg;
using namespace PlayerCc;


/** Proxy structure where data from the ClientProxies are stored.*/
struct RobotState {
   Pose position; 
   std::vector<ModelLaser::Sample> laser;
   std::vector<ModelRanger::Sensor> ranger;
   std::vector<double> sonar;
};

typedef ClientProxy* (*constructor_t)(PlayerClient *aPc, uint32_t aIndex);
typedef void (*init_func_t)(Model *mod,ClientProxy* proxy);

struct ProxyConfiguration {
   constructor_t constructor;
   std::string modelType;
   stg_model_callback_t callback;
   init_func_t initfunc;
   ProxyConfiguration(constructor_t constructor, std::string type, stg_model_callback_t callback, init_func_t initfunc) {
      this->constructor = constructor;
      this->modelType = type;
      this->callback = callback;
      this->initfunc = initfunc;
   }
   ProxyConfiguration() {};
};

class RobotConnector;

struct UpdateConfiguration {
   ClientProxy* proxy; 
   uint16_t index;
   RobotConnector* connector;
   UpdateConfiguration(ClientProxy* proxy, uint16_t index, RobotConnector* connector) {
      this->proxy = proxy;
      this->index = index;
      this->connector = connector;
   }
};   

class RobotConnector {
private:
   
   std::vector<UpdateConfiguration*> proxies;
   std::set<std::string> interfaces;
   
   PlayerCc::PlayerClient *robot;
   std::map<int,RobotState> state;
   std::map<Model*,bool > used;
   pthread_mutex_t mutex; 
   
   std::string computer;
   int port;
   
   Model* getChildWithType(Model* mod,std::string type);
   void loadParams(std::string args);
   /** Method to be called by a reader thread. */
   static void* readerThread(void* robotConnector);
   
public:
   /** Whether Stage is closing. */
   static bool done;
   /** All readerThreads of all RobotConnectors. */
   static std::vector<pthread_t> threads;
   /** Constructor.*/
   RobotConnector(Model* mod, CtrlArgs* args);
   // @{   
/** Init callbacks */
   static void LaserInit(Model *mod, ClientProxy *proxy);
   static void SonarInit(Model *mod, ClientProxy *proxy);
   static void RangerInit(Model *mod, ClientProxy *proxy);
   // @}   
   // @{   
/** Update callback */
   static int PositionUpdate( Model* mod, void* dummy );
   static int LaserUpdate( Model* mod, void* dummy );
   static int SonarUpdate( Model* mod, void* pair );
   static int RangerUpdate( Model* mod, void* pair );
   // @}   
   /** Main controller loop. */
   void Run();   
};


#endif