#include "image.hh"

//#include <GL/glut.h>    // Header File For The GLUT Library 
#include <GL/gl.h>   // Header File For The OpenGL32 Library
//#include <GL/glu.h>  // Header File For The GLu32 Library
#include <stdio.h>      // Header file for standard file i/o.
#include <stdlib.h>     // Header file for malloc/free.
//#include <unistd.h>     // needed to sleep.


int ImageLoad(char *filename, Image *image) {
  FILE *file;
  unsigned long size;                 // size of the image in bytes.
  unsigned long i;                    // standard counter.
  unsigned short int planes;          // number of planes in image (must be 1) 
  unsigned short int bpp;             // number of bits per pixel (must be 24)
  char temp;                          // temporary color storage for bgr-rgb conversion.
  
  // make sure the file is there.
  if ((file = fopen(filename, "rb"))==NULL)
  {
    printf("File Not Found : %s\n",filename);
    return 0;
  }
  
  // seek through the bmp header, up to the width/height:
  fseek(file, 18, SEEK_CUR);
  
  // read the width
  if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
    printf("Error reading width from %s.\n", filename);
    return 0;
  }
  printf("Width of %s: %lu\n", filename, image->sizeX);
  
  // read the height 
  if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
    printf("Error reading height from %s.\n", filename);
    return 0;
  }
  printf("Height of %s: %lu\n", filename, image->sizeY);
  
  // calculate the size (assuming 24 bits or 3 bytes per pixel).
  size = image->sizeX * image->sizeY * 3;
  
  // read the planes
  if ((fread(&planes, 2, 1, file)) != 1) {
    printf("Error reading planes from %s.\n", filename);
    return 0;
  }
  if (planes != 1) {
    printf("Planes from %s is not 1: %u\n", filename, planes);
    return 0;
  }
  
  // read the bpp
  if ((i = fread(&bpp, 2, 1, file)) != 1) {
    printf("Error reading bpp from %s.\n", filename);
    return 0;
  }
  if (bpp != 24) {
    printf("Bpp from %s is not 24: %u\n", filename, bpp);
    return 0;
  }
  
  // seek past the rest of the bitmap header.
  fseek(file, 24, SEEK_CUR);
  
  // read the data. 
  image->data = (char *) malloc(size);
  if (image->data == NULL) {
    printf("Error allocating memory for color-corrected image data");
    return 0;   
  }
  
  if ((i = fread(image->data, size, 1, file)) != 1) {
    printf("Error reading image data from %s.\n", filename);
    return 0;
  }
  
  for (i=0;i<size;i+=3) { // reverse all of the colors. (bgr -> rgb)
    temp = image->data[i];
    image->data[i] = image->data[i+2];
    image->data[i+2] = temp;
  }
  
  // we're done.
  return 1;
}


void LoadGLTextures(char *filename, GLuint *texture) { 
  // Load Texture
  Image *image1;
  
  // allocate space for texture
  image1 = (Image *) malloc(sizeof(Image));
  if (image1 == NULL) {
    printf("Error allocating space for image");
    exit(0);
  }
  
  //    if (!ImageLoad("NeHe.bmp", image1)) {
    if (!ImageLoad(filename, image1)) {
      exit(1);
    }        
    
    // Create Texture   
    glGenTextures(1, texture);
//    printf("bind %d\n",*texture);
    glBindTexture(GL_TEXTURE_2D, *texture);   // 2d texture (x and y size)
    
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture
    
    // 2d texture, level of detail 0 (normal), 3 components (red, green, blue), x size from image, y size from image, 
    // border 0 (normal), rgb color data, unsigned byte data, and finally the data itself.
    glTexImage2D(GL_TEXTURE_2D, 0, 3, image1->sizeX, image1->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image1->data);
  };
  
  
  void displayTexture(GLuint texture)
  {
    float xmin = -1;//video_window.pose.x; //todo probably not needed.
    float ymin = -1;//video_window.pose.y;
    float xmax = 1;//640;//video_window.pose.x + video_window.size.x;
    float ymax = 1;//480;//video_window.pose.y + video_window.size.y;
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear The Screen And The Depth Buffer
//    glLoadIdentity();				// Reset The View
    
    glBindTexture(GL_TEXTURE_2D, texture);   // choose the texture to use.
//    printf("drawing texture %i\n",texture);
    
    
    glMatrixMode (GL_PROJECTION);
    glPushMatrix(); //save old projection
    glLoadIdentity ();
    glOrtho( -1, 1, -1, 1, 0, 1 );
    glMatrixMode (GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
// glTranslatef(12.854358/396.854358, -38.925170,0/294.925170);   
//glTranslatef(-0.03239,0.13198+0.02,0);
    GLint viewMat[4];
    glGetIntegerv(GL_VIEWPORT,viewMat);
  glEnable(GL_TEXTURE_2D);
   glDisable( GL_DEPTH_TEST );

///  glTranslatef(-1.5*viewMat[0]/viewMat[2],-1.5*viewMat[1]/viewMat[3],0);
 glBegin(GL_QUADS);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  0.0f);	// Bottom Left Of The Texture and Quad
    glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  0.0f);	// Bottom Right Of The Texture and Quad
    glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  0.0f);	// Top Right Of The Texture and Quad
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  0.0f);	// Top Left Of The Texture and Quad
    glEnd();
  glDisable(GL_TEXTURE_2D);
     glEnable( GL_DEPTH_TEST );
  
    //restore camera projection
    glPopMatrix();
    glMatrixMode (GL_PROJECTION);
    glPopMatrix();
    glMatrixMode (GL_MODELVIEW);
  }    