#ifndef MODEL_OCC_GRID_H  
#define MODEL_OCC_GRID_H  

#include "stage.hh"

using namespace Stg;

struct Pixel {
   uint32_t x;
   uint32_t y;
   Pixel(uint32_t x, uint32_t y) : x(x), y(y) {};
   Pixel() {};
   friend  bool operator< (const Pixel &px1, const Pixel &px2) {
      return px1.x < px2.x || (px1.x == px2.x && px1.y < px2.y);
   }
};


struct DepthColor {
   uint8_t depth;
   Color color;
   DepthColor(uint8_t depth, Color color) : depth(depth), color(color) {};
   DepthColor() {};
};

typedef std::vector<Pixel> PixelVector;

struct ColoredPixels {
   Color color;
   uint8_t type;
   uint8_t depth;
   std::vector<Pixel> pixels;
   ColoredPixels(const Color color, uint8_t type, uint8_t depth, const std::vector<Pixel> pixels) : color(color), type(type), depth(depth), pixels(pixels) {};
   ColoredPixels() : type(-1) {};
};

struct GCell {
   unsigned char value;
   bool draw;
   GCell(unsigned char value=0, bool draw=true) : value(value), draw(draw) {};
};

/// %ModelLaser class
class ModelOccGrid : public Model
{
   public:
      unsigned int rows;
      unsigned int columns;
      GCell **cells;
      std::map<const std::string, ColoredPixels> pixels;
      std::map<Pixel,DepthColor> depths;
     
      /** Convenience object for setting parameters using SetConfig/GetConfig */
      
      class Config
      {
         public:
           float cell_size; // size of the cell for drawing (in centimeters 
           float alpha_color;
      };


   private:	 
      class Vis : public Visualizer 
      {
         private:
            static Option showArea;
            static Option showStrikes;
            static Option showFov;
            static Option showBeams;

         public:
            Vis( World* world );		virtual ~Vis( void ){}
            virtual void Visualize( Model* mod, Camera* cam );
         virtual void DrawPixels(Model* mod, ColoredPixels pixels);
         virtual void SetPixelsDepth(Model* mod, ColoredPixels pixels);
      } vis;
         
         // set up data buffers after the config changes
      void SampleConfig();

   public:
      float cell_size; // size of the cell for drawing (in meters)
      float alpha_color;
         // constructor
      ModelOccGrid( World* world,
                  Model* parent,
                  const std::string& type ); 

         // destructor
      ~ModelOccGrid();
      
   
      virtual void Startup();
      virtual void Shutdown();
      virtual void Update();
      virtual void Load();
      virtual void Print( char* prefix );
      virtual void ReallocCells(unsigned int icols, unsigned int irows);
      virtual void SetCells(unsigned char *icells);
   virtual void AddPixels(const std::string name,  const Color &color, const uint8_t type, const uint8_t depth, const std::vector<Pixel> &pixels);

      
      /** Get the user-tweakable configuration of the model */
      Config GetConfig( );
         
      /** Set the user-tweakable configuration of the model */
      void SetConfig( Config& cfg );  
};
#endif