/*
 * File name: radial_distortion.cc
 * Date:      2010/08/04 13:46
 * Author:    Jan Faigl & TK
 *
 */

#include <cmath>

#include "radial_distortion.hh"

using namespace imr;

/*
static float kk[3][3] = {
   {6.8351203e-06,   7.1654845e-04,  -2.2494381e-02},
   {-7.1219540e-04,   9.7728357e-06,   9.9974643e-01},
   {7.1627526e-10,   2.6849912e-09,   2.4598495e-04}
};
*/

/// - constructor --------------------------------------------------------------
CRadialDistortion::CRadialDistortion(int w, int h, int depth, int cx, int cy, float rho) :
   W(w), H(h), DEPTH(depth), CX(cx), CY(cy), RHO(rho) {
      xArray = new float[W*H];
      yArray = new float[W*H];
      gArrayX = new float[W*H];
      gArrayY = new float[W*H];
      pArray = new int[W*H];

      float ix,iy; 
      float gx,gy,ux,uy;
      for (int x = 0;x<W;x++){
	 for (int y = 0;y<H;y++){
	    xArray[y*W+x] = unbarrelX(x,y);
	    yArray[y*W+x] = unbarrelY(x,y);
	    if (xArray[y*W+x] < 0 || xArray[y*W+x] > W-1 || yArray[y*W+x] < 0 || yArray[y*W+x] > H-1){
	       xArray[y*W+x] = 0; 
	       yArray[y*W+x] = 0; 
	    }	
	    ux = trunc(xArray[y*W+x]);
	    uy = trunc(yArray[y*W+x]);
	    gx = xArray[y*W+x]-ux;
	    gy = yArray[y*W+x]-uy;
	    ix = (int)ux;
	    iy = (int)uy;
	    pArray[y*W+x] = W*iy+ix;
	    gArrayX[y*W+x] = gx; 
	    gArrayY[y*W+x] = gy; 
	 }
      }
   }

/// - destructor ---------------------------------------------------------------
CRadialDistortion::~CRadialDistortion() {
   delete[] xArray;
   delete[] yArray;
   delete[] gArrayX;
   delete[] gArrayY;
   delete[] pArray;
}

/// - public method ------------------------------------------------------------
void CRadialDistortion::transform(unsigned char *dst, unsigned char *src) const {
   src[0] = 0;
   for(int p = 0; p < (W*(H-1)); p++) {
      const float& gx = gArrayX[p];
      const float& gy = gArrayY[p];
      for (int i = 0; i < 3; i++) {
	 dst[DEPTH*p+i] = src[DEPTH*pArray[p] + i]*(1-gx)*(1-gy)+src[DEPTH*(pArray[p]+1) + i]*gx*(1-gy)+src[DEPTH*(pArray[p]+W) +i]*(1-gx)*gy+src[DEPTH*(pArray[p]+W+1)+i]*gx*gy; 
      }
   }
}

/// - private method -----------------------------------------------------------
float CRadialDistortion::unbarrelX(int x,int y) {
   x = x - CX;
   y = y - CY;
   const float r = x*x+y*y;
   return x*(1-r*RHO)+CX;
}

float CRadialDistortion::unbarrelY(int x,int y) {
   x = (x - CX);
   y = (y - CY);
   const float r = x*x+y*y;
   return y*(1-r*RHO)+CY;
}

/* end of radial_distortion.cc */
