/*
 *  Player - One Hell of a Robot Server
 *  Copyright (C) 2004, 2005 Richard Vaughan
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * Desc: A plugin driver for Player that gives access to Stage devices.
 * Author: Miroslav Kulich
 * Date: 1 October 2009
 */

// DOCUMENTATION ------------------------------------------------------------

/** @addtogroup player
@par Occupancy Grid interface
- PLAYER_LASER_DATA_SCAN
- PLAYER_LASER_REQ_SET_CONFIG
- PLAYER_LASER_REQ_GET_CONFIG
- PLAYER_LASER_REQ_GET_GEOM
*/

// CODE ----------------------------------------------------------------------

#include "p_driver.h"
using namespace Stg;

InterfaceOccGrid::InterfaceOccGrid( player_devaddr_t addr,
				StgDriver* driver,
				ConfigFile* cf,
				int section )
  : InterfaceModel( addr, driver, cf, section, "occgrid" )
{
}

void InterfaceOccGrid::Publish( void )
{
   ModelOccGrid* mod = (ModelOccGrid*)this->mod;
   (void) mod;
}

int InterfaceOccGrid::ProcessMessage(QueuePointer & resp_queue,
				   player_msghdr_t* hdr,
				   void* data)
{
   ModelOccGrid* mod = (ModelOccGrid*)this->mod;
  // Is it a request to set the laser's config?
  if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD,
                           PLAYER_BITMAP_CMD_COLORS,
                           this->addr))
  {
     player_bitmap_data_t *ddd = (player_bitmap_data_t *) data;
     if (mod->columns != ddd->columns || mod->rows != ddd->rows) {
        mod->ReallocCells(ddd->columns,ddd->rows); 
     }
     mod->SetCells(ddd->cells);
     return(0);
  } 
   
  if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD,
                            PLAYER_BITMAP_CMD_PIXELS,
                            this->addr))
   {
      player_bitmap_pixels_t *ddd = (player_bitmap_pixels_t *) data;
      std::vector<Pixel> pixels;
/*      printf("occgrid name %d\n",ddd->name_count);
      printf("occgrid color %d\n",ddd->color_count);
      printf("occgrid %s %d\n",ddd->name, ddd->pixels_count);*/
      
      for(uint i=0;i<ddd->pixels_count;i++) {
         pixels.push_back(Pixel(ddd->pixels[i].x,ddd->pixels[i].y));
      }
      mod->AddPixels(std::string(ddd->name), Color(ddd->color),ddd->type,ddd->depth,pixels);
      return(0);
   }    
  // Don't know how to handle this message.
  PRINT_WARN2( "stage laser doesn't support message %d:%d.",
	       hdr->type, hdr->subtype);
  return(-1);
}


